@@ -1,5 +1,5 @@
 /*
- * gpe-mini-browser v0.16
+ * gpe-mini-browser v0.17
  *
  * Basic web browser based on gtk-webcore 
  *
@@ -47,54 +47,51 @@
 
 //#define DEBUG /* uncomment this if you want debug output*/
 
-static int fullscreen = 0;
-
-static void
-set_fullscreen (GtkWidget * button, GtkWidget * app)
-{
-  if (!fullscreen)
-    {
-      gtk_window_fullscreen (GTK_WINDOW (app));
-      fullscreen = 1;
-    }
-  else
-    {
-      gtk_window_unfullscreen (GTK_WINDOW (app));
-      fullscreen = 0;
-    }
-}
 
+struct gpe_icon my_icons[] = {
+  {"gpe-mini-browser-icon", PREFIX "/share/pixmaps/gpe-mini-browser.png"},
+  {NULL, NULL}
+};
 
 int
 main (int argc, char *argv[])
 {
   GtkWidget *html, *app, *contentbox;	/* html engine, application window, content box of application window */
   GtkWidget *toolbar, *urlbox = NULL;	/* toolbar, url entry box (big screen) */
-  GtkToolItem *back_button, *forward_button, *home_button, *bookmarks_button,
-    *fullscreen_button, *url_button = NULL;
-  GtkToolItem *separator, *separator2, *separator3;
+  GtkToolItem *back_button, *forward_button, *home_button,
+    *fullscreen_button, *url_button = NULL, *history_button;
+#ifndef NOBOOKMARKS
+  GtkToolItem *bookmarks_button, *separator;
+#endif /* NOBOOKMARKS */
+  GtkToolItem *separator2, *separator3;
   extern GtkToolItem *stop_reload_button;
   const gchar *base;
   gint width = 240, height = 320;
   static struct status_data status;
+  static struct fullscreen_info fsinfo;
   int opt;
 
   WebiSettings s = { 0, };
 
   gpe_application_init (&argc, &argv);
 
+  if (gpe_load_icons (my_icons) == FALSE)
+    exit (1);
+
   while ((opt = getopt (argc, argv, "vh")) != -1)
     {
       switch (opt)
 	{
 	case 'v':
 	  printf
-	    (_("GPE-mini-browser version 0.16. (C)2005, Philippe De Swert\n"));
+	    (_
+	     ("GPE-mini-browser version 0.17. (C)2005, Philippe De Swert\n"));
 	  exit (0);
 
 	default:
 	  printf
-	    (_("GPE-mini-browser, basic web browser application. (c)2005, Philippe De Swert\n"));
+	    (_
+	     ("GPE-mini-browser, basic web browser application. (c)2005, Philippe De Swert\n"));
 	  printf (_("Usage: gpe-mini-browser <URL>\n"));
 	  printf (_("Use -v for version info.\n"));
 	  exit (0);
@@ -116,18 +113,24 @@ main (int argc, char *argv[])
   height = gdk_screen_height ();
   gtk_window_set_title (GTK_WINDOW (app), "mini-browser");
   gtk_window_set_default_size (GTK_WINDOW (app), width, height);
+  gpe_set_window_icon (app, "gpe-mini-browser-icon");
 
   //create boxes
   contentbox = gtk_vbox_new (FALSE, 0);
 
+  //create toolbar and add to topbox
+  toolbar = gtk_toolbar_new ();
+
   //fill in status to be sure everything is filled in when used
   //status = (struct status_data*) malloc (sizeof (struct status_data));
   status.main_window = contentbox;
   status.pbar = NULL;
   status.exists = FALSE;
 
-  //create toolbar and add to topbox
-  toolbar = gtk_toolbar_new ();
+  /* fill in fullscreen info */
+  fsinfo.app = app;
+  fsinfo.toolbar = toolbar;
+  fsinfo.urlbox = NULL;		/* set to NULL for small screen interface. Will be filled in when we actually have an urlbox */
 
   //create html object (must be created before function calls to html to avoid segfault)
   html = webi_new ();
@@ -147,7 +150,7 @@ main (int argc, char *argv[])
 
   /* Connect all the signals to the rendering object */
   /* cookies will only decently work when fixed in gtk-webcore */
-  g_signal_connect (WEBI (html), "set_cookie", 
+  g_signal_connect (WEBI (html), "set_cookie",
 		    G_CALLBACK (handle_cookie), NULL);
 
   g_signal_connect (WEBI (html), "load_start",
@@ -156,11 +159,10 @@ main (int argc, char *argv[])
   g_signal_connect (WEBI (html), "load_stop",
 		    G_CALLBACK (destroy_status_window), &status);
 
-  g_signal_connect (WEBI (html), "status", 
-                    G_CALLBACK (activate_statusbar), &status);
+  g_signal_connect (WEBI (html), "status",
+		    G_CALLBACK (activate_statusbar), &status);
 
-  g_signal_connect (WEBI (html), "title", 
- 		    G_CALLBACK (set_title), app);
+  g_signal_connect (WEBI (html), "title", G_CALLBACK (set_title), app);
 
   /*add home,  back, forward, refresh / stop, url (small screen) */
   back_button = gtk_tool_button_new_from_stock (GTK_STOCK_GO_BACK);
@@ -179,36 +181,38 @@ main (int argc, char *argv[])
   gtk_tool_item_set_homogeneous (home_button, FALSE);
   gtk_toolbar_insert (GTK_TOOLBAR (toolbar), home_button, -1);
 
-  separator2 = gtk_separator_tool_item_new ();
-  gtk_tool_item_set_homogeneous (separator2, FALSE);
-  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), separator2, -1);
-
 #ifndef NOBOOKMARKS
+  separator = gtk_separator_tool_item_new ();
+  gtk_tool_item_set_homogeneous (separator, FALSE);
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), separator, -1);
+
   bookmarks_button = gtk_tool_button_new_from_stock (GTK_STOCK_INDENT);
   gtk_tool_item_set_homogeneous (bookmarks_button, FALSE);
   gtk_tool_button_set_label (GTK_TOOL_BUTTON (bookmarks_button),
 			     _("Bookmarks"));
   gtk_toolbar_insert (GTK_TOOLBAR (toolbar), bookmarks_button, -1);
 
-  separator = gtk_separator_tool_item_new ();
-  gtk_tool_item_set_homogeneous (separator, FALSE);
-  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), separator, -1);
 #endif
 
   /* only show full Url bar if the screen is bigger than 240x320 | 320x240 */
   if ((width > 320) || (height > 320))
     {
       urlbox = show_big_screen_interface (WEBI (html), toolbar, &s);
+      fsinfo.urlbox = urlbox;
     }
   else
     {
+      /* create list here so that bookmarks will also be listed in the history 
+         before the entrycompletion is activated when the urlentry is created for the first time */
+      completion_store = gtk_list_store_new (1, G_TYPE_STRING);
+
       url_button = gtk_tool_button_new_from_stock (GTK_STOCK_NETWORK);
       gtk_tool_item_set_homogeneous (url_button, FALSE);
       gtk_tool_button_set_label (GTK_TOOL_BUTTON (url_button), "Url");
       gtk_toolbar_insert (GTK_TOOLBAR (toolbar), url_button, -1);
-      separator3 = gtk_separator_tool_item_new ();
-      gtk_tool_item_set_homogeneous (separator3, FALSE);
-      gtk_toolbar_insert (GTK_TOOLBAR (toolbar), separator3, -1);
+      separator2 = gtk_separator_tool_item_new ();
+      gtk_tool_item_set_homogeneous (separator2, FALSE);
+      gtk_toolbar_insert (GTK_TOOLBAR (toolbar), separator2, -1);
 
       g_signal_connect (GTK_OBJECT (url_button), "clicked",
 			G_CALLBACK (show_url_window), html);
@@ -223,6 +227,22 @@ main (int argc, char *argv[])
 			     _("Fullscreen"));
   gtk_toolbar_insert (GTK_TOOLBAR (toolbar), fullscreen_button, -1);
 
+  /* add history button and separator */
+  separator3 = gtk_separator_tool_item_new ();
+  gtk_tool_item_set_homogeneous (separator3, FALSE);
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), separator3, -1);
+  /* add zoom to small-screen interface */
+  if ((width < 320) || (height < 320))
+    {
+      add_zoom_buttons (WEBI (html), toolbar, &s);
+    }
+
+  history_button = gtk_tool_button_new_from_stock (GTK_STOCK_HARDDISK);
+  gtk_tool_item_set_homogeneous (history_button, FALSE);
+  gtk_tool_button_set_label (GTK_TOOL_BUTTON (history_button), _("History"));
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), history_button, -1);
+
+
   /* connect all button signals */
   g_signal_connect (GTK_OBJECT (back_button), "clicked",
 		    G_CALLBACK (back_func), html);
@@ -237,11 +257,17 @@ main (int argc, char *argv[])
 		    G_CALLBACK (home_func), html);
 
   g_signal_connect (GTK_OBJECT (fullscreen_button), "clicked",
-		    G_CALLBACK (set_fullscreen), app);
+		    G_CALLBACK (set_fullscreen), &fsinfo);
 #ifndef NOBOOKMARKS
   g_signal_connect (GTK_OBJECT (bookmarks_button), "clicked",
 		    G_CALLBACK (show_bookmarks), html);
 #endif
+  g_signal_connect (GTK_OBJECT (history_button), "clicked",
+		    G_CALLBACK (show_history), html);
+
+  /* save completion list when we exit the program */
+  g_signal_connect (GTK_OBJECT (app), "destroy",
+		    G_CALLBACK (save_completion), NULL);
 
   /* 
      DEBUG CODE!
