@@ -1,5 +1,5 @@
 /*
- * gpe-mini-browser v0.15
+ * gpe-mini-browser v0.17
  *
  * Basic web browser based on gtk-webcore.
  * Hildon interface version for Maemo / Nokia 770
@@ -51,21 +51,6 @@
 
 //#define DEBUG /* uncomment this if you want debug output*/
 
-
-static void
-fullscreen (GtkWidget * button, HildonAppView * app)
-{
-  gboolean fullscreen;
-
-  fullscreen = hildon_appview_get_fullscreen (app);
-  if (!fullscreen)
-    hildon_appview_set_fullscreen (app, TRUE);
-  else
-    hildon_appview_set_fullscreen (app, FALSE);
-
-}
-
-
 static void
 osso_top_callback (const gchar * arguments, gpointer ptr)
 {
@@ -84,18 +69,20 @@ main (int argc, char *argv[])
 
   GtkWidget *html, *contentbox;	/* html engine, application window, content box of application window */
   GtkWidget *toolbar, *urlbox;	/* toolbar, url entry box (big screen), icon for url pop-up window (small screens) */
-  GtkToolItem *back_button, *forward_button, *home_button, *fullscreen_button;
+  GtkToolItem *back_button, *forward_button, *home_button, *fullscreen_button,
+    *bookmarks_button;
   GtkToolItem *separator;
   extern GtkToolItem *stop_reload_button;
   const gchar *base;
-  struct status_data *status;
+  static struct status_data status;
+  static struct fullscreen_info fsinfo;
   int opt;
   osso_context_t *context;
 
   WebiSettings s = { 0, };
 
   /* osso stuff  */
-  context = osso_initialize ("gpe-mini-browser", "0.15", TRUE, NULL);
+  context = osso_initialize ("gpe-mini-browser", "0.17", TRUE, NULL);
   if (context == NULL)
     {
       fprintf (stderr, "osso_initialize failed.\n");
@@ -110,14 +97,16 @@ main (int argc, char *argv[])
 	{
 	case 'v':
 	  printf
-	    ("GPE-mini-browser version 0.14. (C)2005, Philippe De Swert\n");
+	    (_
+	     ("GPE-mini-browser version 0.17. (C)2005, Philippe De Swert\n"));
 	  exit (0);
 
 	default:
 	  printf
-	    ("GPE-mini-browser, basic web browser application. (c)2005, Philippe De Swert\n");
-	  printf ("Usage: gpe-mini-browser <URL>\n");
-	  printf ("Use -v for version info.\n");
+	    (_
+	     ("GPE-mini-browser, basic web browser application. (c)2005, Philippe De Swert\n"));
+	  printf (_("Usage: gpe-mini-browser <URL>\n"));
+	  printf (_("Use -v for version info.\n"));
 	  exit (0);
 	}
     }
@@ -148,9 +137,9 @@ main (int argc, char *argv[])
   contentbox = gtk_vbox_new (FALSE, 0);
 
   //fill in status to be sure everything is filled in when used
-  status = malloc (sizeof (struct status_data));
-  status->main_window = contentbox;
-  status->exists = FALSE;
+  status.main_window = contentbox;
+  status.pbar = NULL;
+  status.exists = FALSE;
 
   //create toolbar and add to topbox
   toolbar = gtk_toolbar_new ();
@@ -181,13 +170,13 @@ main (int argc, char *argv[])
 		    NULL);
 
   g_signal_connect (WEBI (html), "load_start",
-		    G_CALLBACK (create_status_window), status);
+		    G_CALLBACK (create_status_window), &status);
 
   g_signal_connect (WEBI (html), "load_stop",
-		    G_CALLBACK (destroy_status_window), status);
+		    G_CALLBACK (destroy_status_window), &status);
 
   g_signal_connect (WEBI (html), "status", G_CALLBACK (activate_statusbar),
-		    status);
+		    &status);
   g_signal_connect (WEBI (html), "title", G_CALLBACK (set_title), app);
 
 /*add home,  back, forward, refresh / stop, url (small screen) */
@@ -207,14 +196,29 @@ main (int argc, char *argv[])
   gtk_tool_item_set_homogeneous (home_button, FALSE);
   gtk_toolbar_insert (GTK_TOOLBAR (toolbar), home_button, -1);
 
+#ifndef NOBOOKMARKS
   separator = gtk_separator_tool_item_new ();
   gtk_tool_item_set_homogeneous (separator, FALSE);
   gtk_toolbar_insert (GTK_TOOLBAR (toolbar), separator, -1);
 
+  bookmarks_button = gtk_tool_button_new_from_stock (GTK_STOCK_INDENT);
+  gtk_tool_item_set_homogeneous (bookmarks_button, FALSE);
+  gtk_tool_button_set_label (GTK_TOOL_BUTTON (bookmarks_button),
+			     _("Bookmarks"));
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), bookmarks_button, -1);
+
+#endif
+
+
   hildon_appview_set_toolbar (mainview, GTK_TOOLBAR (toolbar));
 
   urlbox = show_big_screen_interface (WEBI (html), toolbar, &s);
 
+  /* fill in fullscreen info */
+  fsinfo.app = GTK_WIDGET (app);
+  fsinfo.toolbar = toolbar;
+  fsinfo.urlbox = urlbox;
+
 /* replace GTK_STOCK_ZOOM_FIT with GTK_STOCK_FULLSCREEN once GPE uses
      gtk 2.7.1 or higher. Or add it myself :-) */
   fullscreen_button = gtk_tool_button_new_from_stock (GTK_STOCK_ZOOM_FIT);
@@ -235,7 +239,12 @@ main (int argc, char *argv[])
 		    G_CALLBACK (home_func), html);
 
   g_signal_connect (GTK_OBJECT (fullscreen_button), "clicked",
-		    G_CALLBACK (fullscreen), mainview);
+		    G_CALLBACK (set_fullscreen), &fsinfo);
+
+#ifndef NOBOOKMARKS
+  g_signal_connect (GTK_OBJECT (bookmarks_button), "clicked",
+		    G_CALLBACK (show_bookmarks), html);
+#endif
 
 //  gtk_toolbar_set_icon_size(GTK_TOOLBAR(toolbar), GTK_ICON_SIZE_SMALL_TOOLBAR);
 //  toolbar size seems to be the same, icons are clearer
